@@ -28,9 +28,9 @@ complete_checksums binaries_complete_checksums;
 /* Functions for checkindeb.c and incoming.c: */
 
 struct deb_headers {
-	char *name,*version;
+	char *name, *version;
 	char *source;
-	architecture_t architecture_atom;
+	architecture_t architecture;
 	char *control;
 	/* only extracted when requested: */
 	/*@null@*/char *sourceversion;
@@ -44,14 +44,14 @@ struct deb_headers {
  *   error returned
  * - no checks for sanity of values, left to the caller */
 
-retvalue binaries_readdeb(struct deb_headers *, const char *filename, bool needssourceversion);
+retvalue binaries_readdeb(struct deb_headers *, const char *filename, bool /*needssourceversion*/);
 void binaries_debdone(struct deb_headers *);
 
 retvalue binaries_calcfilekeys(component_t, const struct deb_headers *, packagetype_t, /*@out@*/struct strlist *);
 
 struct overridedata;
-retvalue binaries_complete(const struct deb_headers *, const char *filekey, const struct checksums *, const struct overridedata *, const char *section, const char *priority, char **newcontrol);
+retvalue binaries_complete(const struct deb_headers *, const char * /*filekey*/, const struct checksums *, const struct overridedata *, const char * /*section*/, const char * /*priority*/, char **/*newcontrol_p*/);
 
-retvalue binaries_adddeb(const struct deb_headers *, struct database *, const struct atomlist *forcedarchitectures, packagetype_t, struct distribution *, /*@null@*/struct trackingdata *, component_t, const struct strlist *filekeys, const char *control);
-retvalue binaries_checkadddeb(const struct deb_headers *, struct database *, architecture_t forcearchitecture, packagetype_t, struct distribution *, bool tracking, component_t, bool permitnewerold);
+retvalue binaries_adddeb(const struct deb_headers *, const struct atomlist */*forcedarchitectures*/, packagetype_t, struct distribution *, /*@null@*/struct trackingdata *, component_t, const struct strlist */*filekeys*/, const char */*control*/);
+retvalue binaries_checkadddeb(const struct deb_headers *, architecture_t /*forcearchitecture*/, packagetype_t, struct distribution *, bool tracking, component_t, bool /*permitnewerold*/);
 #endif
