@@ -17,7 +17,7 @@ retvalue pool_markadded(const char *);
 retvalue pool_markdeleted(const char *);
 
 /* Remove all files that lost their last reference, or only count them */
-retvalue pool_removeunreferenced(bool delete);
+retvalue pool_removeunreferenced(bool /*delete*/);
 
 /* Delete all added files that are not used, or only count them */
 void pool_tidyadded(bool deletenew);
@@ -25,6 +25,9 @@ void pool_tidyadded(bool deletenew);
 /* delete and forget a single file */
 retvalue pool_delete(const char *);
 
+/* notify outhook of new files */
+void pool_sendnewfiles(void);
+
 /* free all memory, to make valgrind happier */
 void pool_free(void);
 #endif
