@@ -18,6 +18,9 @@ struct release;
 enum indexcompression {ic_uncompressed=0, ic_gzip,
 #ifdef HAVE_LIBBZ2
 			ic_bzip2,
+#endif
+#ifdef HAVE_LIBLZMA
+			ic_xz,
 #endif
 			ic_count /* fake item to get count */
 };
