@@ -301,7 +301,7 @@ static retvalue find_key_and_add(struct uploaders *u, struct upload_conditions *
 			c -= 'a' - 'A';
 		else if (c == 'x' && len-i-1 == 1 && fingerprint[0] == '0')
 			break;
-		if ((c < '0' || c > '9') && (c <'A' && c > 'F')) {
+		if ((c < '0' || c > '9') && (c <'A' || c > 'F')) {
 			fprintf(stderr,
 "Strange character '%c'(=%hhu) in fingerprint '%s'.\n"
 "Search for appropriate rules in the uploaders file might fail.\n",
@@ -327,7 +327,7 @@ static retvalue find_key_and_add(struct uploaders *u, struct upload_conditions *
 		else if (c == 'x' && primary_len-i-1 == 1 &&
 				primary_fingerprint[0] == '0')
 			break;
-		if ((c < '0' || c > '9') && (c <'A' && c > 'F')) {
+		if ((c < '0' || c > '9') && (c <'A' || c > 'F')) {
 			fprintf(stderr,
 "Strange character '%c'(=%hhu) in fingerprint/key-id '%s'.\n"
 "Search for appropriate rules in the uploaders file might fail.\n",
@@ -437,7 +437,7 @@ enum upload_condition_type uploaders_nextcondition(struct upload_conditions *c)
 					return uc_ACCEPTED;
 				c->current = c->current->next_if_true;
 			} else {
-				/* empty set fullfills all conditions,
+				/* empty set fulfills all conditions,
 				   but not an exists condition */
 				switch (c->current->needs) {
 					case needs_any:
@@ -922,7 +922,7 @@ static retvalue parse_condition(const struct filebeingparsed *fbp, int column, c
 			/* everything in current scope that made it fail
 			 * now makes it check this: (currently that will
 			 * only be true at most for c == last, but with
-			 * parantheses this all will be needed) */
+			 * parentheses this all will be needed) */
 			for (c = or_scope ; c != NULL ; c = c->next) {
 				if (c->next_if_true == fallback)
 					c->next_if_true = n;
